@@ -42,6 +42,9 @@
 #include "compare_core.h"
 #include "osal_preproc.h"
 
+/* Version number for UI-Console config section parameters */
+#define CONFIG_PARAM_VERSION     1.00
+
 /** global variables **/
 int    g_Verbose = 0;
 
@@ -62,7 +65,7 @@ static const char *l_ROMFilepath = NULL;       // filepath of ROM to load & run
 
 static int  *l_TestShotList = NULL;      // list of screenshots to take for regression test support
 static int   l_TestShotIdx = 0;          // index of next screenshot frame in list
-static int   l_SaveOptions = 0;          // save command-line options in configuration file
+static int   l_SaveOptions = 1;          // save command-line options in configuration file (enabled by default)
 static int   l_CoreCompareMode = 0;      // 0 = disable, 1 = send, 2 = receive
 
 static eCheatMode l_CheatMode = CHEAT_DISABLE;
@@ -111,6 +114,8 @@ static void FrameCallback(unsigned int FrameIndex)
 
 static m64p_error OpenConfigurationHandles(void)
 {
+    float fConfigParamsVersion;
+    int bSaveConfig = 0;
     m64p_error rval;
 
     /* Open Configuration sections for core library and console User Interface */
@@ -135,13 +140,40 @@ static m64p_error OpenConfigurationHandles(void)
         return rval;
     }
 
+    if ((*ConfigGetParameter)(l_ConfigUI, "Version", M64TYPE_FLOAT, &fConfigParamsVersion, sizeof(float)) != M64ERR_SUCCESS)
+    {
+        fprintf(stderr, "Warning: No version number in 'UI-Console' config section. Setting defaults.\n");
+        (*ConfigDeleteSection)("UI-Console");
+        (*ConfigOpenSection)("UI-Console", &l_ConfigUI);
+        bSaveConfig = 1;
+    }
+    else if (((int) fConfigParamsVersion) != ((int) CONFIG_PARAM_VERSION))
+    {
+        fprintf(stderr, "Warning: Incompatible version %.2f in 'UI-Console' config section: current is %.2f. Setting defaults.\n", fConfigParamsVersion, (float) CONFIG_PARAM_VERSION);
+        (*ConfigDeleteSection)("UI-Console");
+        (*ConfigOpenSection)("UI-Console", &l_ConfigUI);
+        bSaveConfig = 1;
+    }
+    else if ((CONFIG_PARAM_VERSION - fConfigParamsVersion) >= 0.0001f)
+    {
+        /* handle upgrades */
+        float fVersion = CONFIG_PARAM_VERSION;
+        ConfigSetParameter(l_ConfigUI, "Version", M64TYPE_FLOAT, &fVersion);
+        fprintf(stderr, "Info: Updating parameter set version in 'UI-Console' config section to %.2f\n", fVersion);
+        bSaveConfig = 1;
+    }
+
     /* Set default values for my Config parameters */
+    (*ConfigSetDefaultFloat)(l_ConfigUI, "Version", CONFIG_PARAM_VERSION,  "Mupen64Plus UI-Console config parameter set version number.  Please don't change");
     (*ConfigSetDefaultString)(l_ConfigUI, "PluginDir", OSAL_CURRENT_DIR, "Directory in which to search for plugins");
     (*ConfigSetDefaultString)(l_ConfigUI, "VideoPlugin", "mupen64plus-video-rice" OSAL_DLL_EXTENSION, "Filename of video plugin");
     (*ConfigSetDefaultString)(l_ConfigUI, "AudioPlugin", "mupen64plus-audio-sdl" OSAL_DLL_EXTENSION, "Filename of audio plugin");
     (*ConfigSetDefaultString)(l_ConfigUI, "InputPlugin", "mupen64plus-input-sdl" OSAL_DLL_EXTENSION, "Filename of input plugin");
     (*ConfigSetDefaultString)(l_ConfigUI, "RspPlugin", "mupen64plus-rsp-hle" OSAL_DLL_EXTENSION, "Filename of RSP plugin");
 
+    if (bSaveConfig && ConfigSaveSection != NULL) /* ConfigSaveSection was added in Config API v2.1.0 */
+        (*ConfigSaveSection)("UI-Console");
+
     return M64ERR_SUCCESS;
 }
 
@@ -180,6 +212,7 @@ static void printUsage(const char *progname)
            "    --fullscreen          : use fullscreen display mode\n"
            "    --windowed            : use windowed display mode\n"
            "    --resolution (res)    : display resolution (640x480, 800x600, 1024x768, etc)\n"
+           "    --nospeedlimit        : disable core speed limiter (should be used with dummy audio plugin)\n"
            "    --cheats (cheat-spec) : enable or list cheat codes for the given rom file\n"
            "    --corelib (filepath)  : use core library (filepath) (can be only filename or full path)\n"
            "    --configdir (dir)     : force configation directory to (dir); should contain mupen64plus.conf\n"
@@ -195,7 +228,7 @@ static void printUsage(const char *progname)
            "    --set (param-spec)    : set a configuration variable, format: ParamSection[ParamName]=Value\n"
            "    --core-compare-send   : use the Core Comparison debugging feature, in data sending mode\n"
            "    --core-compare-recv   : use the Core Comparison debugging feature, in data receiving mode\n"
-           "    --saveoptions         : save the given command-line options in configuration file for future\n"
+           "    --nosaveoptions       : do not save the given command-line options in configuration file\n"
            "    --verbose             : print lots of information\n"
            "    --help                : see this help message\n\n"
            "(plugin-spec):\n"
@@ -391,6 +424,17 @@ static m64p_error ParseCommandLineFinal(int argc, const char **argv)
             int Fullscreen = 0;
             (*ConfigSetParameter)(l_ConfigVideo, "Fullscreen", M64TYPE_BOOL, &Fullscreen);
         }
+        else if (strcmp(argv[i], "--nospeedlimit") == 0)
+        {
+            int EnableSpeedLimit = 0;
+            if (g_CoreAPIVersion < 0x020001)
+                fprintf(stderr, "Warning: core library doesn't support --nospeedlimit\n");
+            else
+            {
+                if ((*CoreDoCommand)(M64CMD_CORE_STATE_SET, M64CORE_SPEED_LIMITER, &EnableSpeedLimit) != M64ERR_SUCCESS)
+                    fprintf(stderr, "Error: core gave error while setting --nospeedlimit option\n");
+            }
+        }
         else if ((strcmp(argv[i], "--corelib") == 0 || strcmp(argv[i], "--configdir") == 0 ||
                   strcmp(argv[i], "--datadir") == 0) && ArgsLeft >= 1)
         {   /* these are handled in ParseCommandLineInitial */
@@ -487,9 +531,9 @@ static m64p_error ParseCommandLineFinal(int argc, const char **argv)
         {
             l_CoreCompareMode = 2;
         }
-        else if (strcmp(argv[i], "--saveoptions") == 0)
+        else if (strcmp(argv[i], "--nosaveoptions") == 0)
         {
-            l_SaveOptions = 1;
+            l_SaveOptions = 0;
         }
         else if (ArgsLeft == 0)
         {
@@ -537,7 +581,7 @@ int main(int argc, char *argv[])
         return 2;
 
     /* start the Mupen64Plus core library, load the configuration file */
-    m64p_error rval = (*CoreStartup)(CONSOLE_API_VERSION, l_ConfigDirPath, l_DataDirPath, "Core", DebugCallback, NULL, NULL);
+    m64p_error rval = (*CoreStartup)(CORE_API_VERSION, l_ConfigDirPath, l_DataDirPath, "Core", DebugCallback, NULL, NULL);
     if (rval != M64ERR_SUCCESS)
     {
         printf("UI-console: error starting Mupen64Plus core library.\n");
@@ -675,7 +719,7 @@ int main(int argc, char *argv[])
     /* close the ROM image */
     (*CoreDoCommand)(M64CMD_ROM_CLOSE, 0, NULL);
 
-    /* save the configuration file again if --saveoptions was specified, to keep any updated parameters from the core/plugins */
+    /* save the configuration file again if --nosaveoptions was not specified, to keep any updated parameters from the core/plugins */
     if (l_SaveOptions)
         SaveConfigurationOptions();
 
